"""
Classifies: CHEBI:16337 phosphatidic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_phosphatidic_acid(smiles: str):
    """
    Determines if a molecule is a phosphatidic acid - a derivative of glycerol with one hydroxy group 
    esterified with phosphoric acid and the other two esterified with fatty acids.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a phosphatidic acid, False otherwise
        str: Reason for classification
    """
    # Parse SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Look for phosphate group (-OP(=O)(O)O)
    phosphate_pattern = Chem.MolFromSmarts('[O,OH]-P(=O)([O,OH])[O,OH]')
    if not mol.HasSubstructMatch(phosphate_pattern):
        return False, "No phosphate group found"

    # Look for glycerol backbone with specific connectivity
    # Pattern matches glycerol backbone with phosphate and two ester groups
    glycerol_pattern = Chem.MolFromSmarts('[#6]C(=O)O[CH2][CH](O[#6])[CH2]OP(=O)([O,OH])[O,OH]')
    
    if not mol.HasSubstructMatch(glycerol_pattern):
        # Try alternative pattern for different representation
        glycerol_pattern2 = Chem.MolFromSmarts('[#6]C(=O)OC[CH]([CH2]OP(=O)([O,OH])[O,OH])OC(=O)[#6]')
        if not mol.HasSubstructMatch(glycerol_pattern2):
            # Try pattern for generic structures with wildcards
            if '*' in smiles:
                generic_pattern = Chem.MolFromSmarts('C(O[#6])(CO[#6])[CH2]OP(=O)([O,OH])[O,OH]')
                if mol.HasSubstructMatch(generic_pattern):
                    return True, "Generic phosphatidic acid structure found"
            return False, "No proper glycerol backbone found"

    # Look for two ester groups (-C(=O)O-) connected to glycerol
    ester_pattern = Chem.MolFromSmarts('C(=O)O[CH2,CH]')
    ester_matches = mol.GetSubstructMatches(ester_pattern)
    if len(ester_matches) < 2:
        return False, "Less than 2 ester groups found"

    # Check for fatty acid chains (at least 4 carbons in length)
    fatty_acid_pattern = Chem.MolFromSmarts('C(=O)O[CH2,CH]-[CH2]-[CH2]-[CH2]')
    if not mol.HasSubstructMatch(fatty_acid_pattern):
        return False, "No proper fatty acid chains found"

    # All criteria met
    return True, "Valid phosphatidic acid structure found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:16337',
                          'name': 'phosphatidic acid',
                          'definition': 'A derivative of glycerol in which one '
                                        'hydroxy group, commonly but not '
                                        'necessarily primary, is esterified '
                                        'with phosphoric acid and the other '
                                        'two are esterified with fatty acids.',
                          'parents': ['CHEBI:37739']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0 is too low.\n'
               'True positives: []\n'
               'False positives: '
               "[('C([C@H](NC(*)=O)C([O-])=O)OP(=O)([O-])OC[C@H](O)COC(=O)*', "
               "'Generic phosphatidic acid structure found'), "
               "('P([O-])(=O)(OC[C@@H](C(=O)[O-])[NH3+])OC[C@@H](COC(=O)*)OC(=O)*', "
               "'Generic phosphatidic acid structure found'), "
               "('[NH3+][C@@H](COP([O-])(=O)OC[C@@H](CO[*])O[*])C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('[NH3+][C@@H](COP([O-])(=O)OCC(COC([*])=O)OC([*])=O)C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('[C@@H](COC(=O)*)(COP(OC[C@@H](C(=O)[O-])[NH3+])(=O)[O-])OC(=O)*', "
               "'Generic phosphatidic acid structure found'), "
               "('[NH3+][C@@H](COP([O-])(=O)OCC(CO)OC([*])=O)C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('[NH3+][C@@H](COP([O-])(=O)OCC(CO[*])O[*])C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('[NH3+][C@@H](COP(O)(=O)OC[C@H](COC([*])=O)OC([*])=O)C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('N[C@@H](COP(O)(=O)OC[C@@H](CO[*])O[*])C(O)=O', 'Generic "
               "phosphatidic acid structure found'), "
               "('[NH3+][C@@H](COP([O-])(=O)OC[C@@H](CO)OC([*])=O)C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('C(=O)(N[C@H](C(=O)O)COP(OC[C@@H](COC(=O)*)OC(=O)*)(=O)O)*', "
               "'Generic phosphatidic acid structure found'), "
               "('P([O-])(=O)(OC[C@@H](C(=O)[O-])NC(C[NH3+])=O)OC[C@@H](COC(=O)*)OC(=O)*', "
               "'Generic phosphatidic acid structure found'), "
               "('[C@](CO/C=C\\\\*)(O)([H])COP(OC[C@@H](C(O)=O)N)(=O)O', "
               "'Generic phosphatidic acid structure found'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCC(=O)O[C@H](COC([*])=O)COP([O-])(=O)OC[C@H]([NH3+])C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('[NH3+][C@@H](COP([O-])(=O)OCC(CO[*])OC([*])=O)C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('N[C@@H](COP(O)(=O)OC[C@@H](CO[*])OC([*])=O)C(O)=O', 'Generic "
               "phosphatidic acid structure found'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCC(=O)O[C@H](COC=C[*])COP([O-])(=O)OC[C@H]([NH3+])C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('[NH3+][C@@H](COP([O-])(=O)OC[C@@H](COC([*])=O)O[*])C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('[NH3+][C@@H](COP([O-])(=O)OC[C@@H](CO[*])OC([*])=O)C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('[NH3+][C@@H](COP([O-])(=O)OC[C@H](COC([*])=O)OC([*])=O)C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('[NH3+][C@@H](COP([O-])(=O)OCC(O)COC=C[*])C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('[C@](CO*)(O)([H])COP(OC[C@@H](C(O)=O)N)(=O)O', 'Generic "
               "phosphatidic acid structure found'), "
               "('[C@@H](COC(=O)*)(COP(OC[C@@H](C(=O)O)N)(=O)O)OC(=O)*', "
               "'Generic phosphatidic acid structure found'), "
               "('[NH3+][C@@H](COP([O-])(=O)OCC(O)COC([*])=O)C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('C([C@H](NC(*)=O)C([O-])=O)OP(=O)([O-])OC[C@H](OC(*)=O)COC(=O)*', "
               "'Generic phosphatidic acid structure found'), "
               "('P(O)(=O)(OC[C@@H](C(=O)O)N)OC[C@@H](CO*)O*', 'Generic "
               "phosphatidic acid structure found'), "
               "('[NH3+][C@@H](COP([O-])(=O)OC[C@H](O)COC=C[*])C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('C(OC[C@H](COP(OC[C@@H](C(O)=O)N)(=O)O)OC(*)=O)=C*', 'Generic "
               "phosphatidic acid structure found'), "
               "('CCCCCC\\\\C=C/CCCCCCCC(=O)O[C@H](COC([*])=O)COP([O-])(=O)OC[C@H]([NH3+])C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('[NH3+][C@@H](COP([O-])(=O)OC[C@H](O)CO\\\\C=C/[*])C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('[NH3+][C@@H](COP([O-])(=O)OC[C@@H](COC([*])=O)OC([*])=O)C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('P(O)(=O)(OC[C@@H](C(=O)O)N)OC[C@@H](COC(=O)*)OC(=O)*', "
               "'Generic phosphatidic acid structure found'), "
               "('N[C@@H](COP(O)(=O)OC[C@@H](CO)OC([*])=O)C(O)=O', 'Generic "
               "phosphatidic acid structure found'), "
               "('N[C@@H](COP(O)(=O)OC[C@@H](COC([*])=O)O[*])C(O)=O', 'Generic "
               "phosphatidic acid structure found'), "
               "('Nc1ncnc2n(cnc12)[C@@H]1O[C@H](COP([O-])(-*)=O)[C@@H](OC(=O)[C@@H]([NH3+])COP([O-])([O-])=O)[C@H]1O', "
               "'Generic phosphatidic acid structure found'), "
               "('CCCCC\\\\C=C/C\\\\C=C/CCCCCCCC(=O)O[C@H](COC([*])=O)COP([O-])(=O)OC[C@H]([NH3+])C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('O(P(OC[C@@H](C(=O)O)N)(=O)O)C[C@@H](CO*)O*', 'Generic "
               "phosphatidic acid structure found'), "
               "('O(C[C@@H](C(=O)[O-])[NH3+])P(=O)(OC[C@@H](COC(=O)*)O)[O-]', "
               "'Generic phosphatidic acid structure found'), "
               "('N[C@@H](COP(O)(=O)OC[C@H](O)COC([*])=O)C(O)=O', 'Generic "
               "phosphatidic acid structure found'), "
               "('O(P(=O)(OC[C@@H](C([O-])=O)NC(CNC(=O)[C@@H](N*)*)=O)[O-])C[C@H](OC(*)=O)COC(*)=O', "
               "'Generic phosphatidic acid structure found'), "
               "('[NH3+][C@@H](COP([O-])(=O)OC[C@H](O)CO[*])C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('[NH3+][C@@H](COP([O-])(=O)OC[C@@H](CO\\\\C=C/[*])OC([*])=O)C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('P(=O)(OC[C@@H](C(=O)[O-])[NH3+])(OC[C@@H](COC(=O)*)OC(=O)*)[O-]', "
               "'Generic phosphatidic acid structure found'), "
               "('[NH3+][C@@H](COP([O-])(=O)OCC(O)CO[*])C([O-])=O', 'Generic "
               "phosphatidic acid structure found'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCC(=O)O[C@H](CO\\\\C=C/[*])COP([O-])(=O)OC[C@H]([NH3+])C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('O(P(=O)(OC[C@@H](C([O-])=O)NC(CN*)=O)[O-])C[C@H](OC(*)=O)COC(*)=O', "
               "'Generic phosphatidic acid structure found'), "
               "('N[C@@H](COP(O)(=O)OC[C@@H](COC([*])=O)OC([*])=O)C(O)=O', "
               "'Generic phosphatidic acid structure found'), "
               "('[NH3+][C@@H](COP([O-])(=O)OCC(COC=C[*])OC([*])=O)C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCC(=O)O[C@H](COC([*])=O)COP([O-])(=O)OC[C@H]([NH3+])C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('C(C(COP(=O)(OC[C@@H](C(=O)O)N)O)OC(=O)*)OC(=O)*', 'Generic "
               "phosphatidic acid structure found'), "
               "('N[C@@H](COP(O)(=O)OC[C@H](COC([*])=O)OC([*])=O)C(O)=O', "
               "'Generic phosphatidic acid structure found'), "
               "('[NH3+][C@@H](COP([O-])(=O)OC[C@@H](COC=C[*])OC([*])=O)C([O-])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('CC(C)(COP(O)(=O)OC[C@H](N)C(O)=O)[C@@H](O)C(=O)NCCC(=O)NCCSC([*])=O', "
               "'Generic phosphatidic acid structure found'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCC(=O)OC[C@H](COP([O-])(=O)OC[C@H]([NH3+])C([O-])=O)OC([*])=O', "
               "'Generic phosphatidic acid structure found')]\n"
               'False negatives: '
               "[('P(OC[C@H](OC(=O)CCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)COC(=O)CCCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCC/C=C\\\\CCCCCCC)COC(=O)CCCCCCCCCCCCCCCCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)COC(=O)CCCCCCCCCCCCCCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('CCCCCCCCCCCCCCCCC(=O)OC[C@H](COP(O)(O)=O)OC(=O)CCCCCCC\\\\C=C/CCCCCCCC', "
               "'No proper glycerol backbone found'), "
               "('CCCCCCCCCCCCCCCC(=O)OC[C@H](COP(O)(O)=O)OC(=O)CCCCCCC\\\\C=C/C\\\\C=C/CCCCC', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCC)COC(=O)CCCCCCCCCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCC/C=C\\\\CCCCCC)COC(=O)CCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('CCCCCCCCCCCCCCCCCC(=O)OC[C@H](COP(O)(O)=O)OC(=O)CCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCC', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCCCCC)COC(=O)CCCCCCC/C=C\\\\CCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCCCCCC)COC(=O)CCCCCCCCCCCCCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCC)COC(=O)CCCCCCCCCCC/C=C\\\\C/C=C\\\\CCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('CCCCCCCCCCCCCCCC(=O)OC[C@H](COP(O)(O)=O)OC(=O)CCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCC', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCC)COC(=O)CCCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCC)COC(=O)CCCCCCCCCCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCCCCC)COC(=O)CCCCCCC/C=C\\\\CCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCC(O)/C=C/C=O)COC(=O)CCCCCCCCCCCCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCC/C=C\\\\C/C=C\\\\CCCCC)COC(=O)CCCCCCC/C=C\\\\CCCCCCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCCC)COC(=O)CCCCCCCCCCCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('[C@@H](COC(=O)*)(COP(=O)(O)O)OC(*)=O', 'No proper glycerol "
               "backbone found'), "
               "('CCCCCCCCCC(=O)OC[C@H](COP(O)(O)=O)OC(=O)CCCCCCCCC', 'No "
               "proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCC)COC(=O)CCCCCCC/C=C\\\\CCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCCCC)COC(=O)CCCCCCCCCCCCCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)COC(=O)CCCCCCCCCCC/C=C\\\\C/C=C\\\\CCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCC(=O)/C=C/C(O)=O)COC(=O)CCCCCCCCCCCCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('O(C(*)=O)C(COC(=O)*)COP(=O)(O)O', 'No proper glycerol "
               "backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCC)COC(=O)CCCCCCCCC/C=C\\\\C/C=C\\\\CCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCCCCCC(C)C)COC(=O)CCCCCCCCCCCCCCCCCCCCC(C)C)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCCCC)COC(=O)CCCCCCCCCCC/C=C\\\\C/C=C\\\\CCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('CCCCCCCCCCCCCCCCCCCC(=O)O[C@H](COC(=O)CCCCCCC\\\\C=C/CCCCCCCC)COP(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)COC(=O)CCCCCCCCCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCC)COC(=O)CCCCCCCCCCCCCCCCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('O(C(*)=O)C(COC(=O)*)COP(=O)(O)O', 'No proper glycerol "
               "backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCC/C=C\\\\C/C=C\\\\CCCCC)COC(=O)CCCCCCCCC/C=C\\\\CCCCCCCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCC[C@H](O)[C@@H](O)C/C=C\\\\CCCCC)COC(=O)CCCCCCCCCCCCC/C=C\\\\CCCCCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCC/C=C\\\\C/C=C\\\\CCCCC)COC(=O)CCCCCCC/C=C\\\\CCCCCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCC/C=C\\\\CCCCCCCC)COC(=O)CCCCCCCCCCCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCC)COC(=O)CCCCCCCCCCCCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)COC(=O)CCCCCCCCCCCCCCCCCCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)COC(=O)CCCCCCC/C=C\\\\C/C=C\\\\CCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('CCCCCCCCCCCCCCCCC(=O)OC[C@H](COP(O)(O)=O)OC(=O)CCCCCCCCCCC', "
               "'No proper glycerol backbone found'), "
               "('O(C(*)=O)C(COC(=O)*)COP(=O)(O)O', 'No proper glycerol "
               "backbone found'), "
               "('C([C@@](COC(CCCCCCCCC/C=C\\\\CCCCCC)=O)(OC(CCCCCCC/C=C\\\\CCCCCCCC)=O)[H])OP(O)(=O)O', "
               "'No proper glycerol backbone found'), "
               "('O(C(*)=O)C(COC(=O)*)COP(=O)(O)O', 'No proper glycerol "
               "backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCC/C=C\\\\C/C=C\\\\CCCCC)COC(=O)CCCCCCCCCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCCCCCC)COC(=O)CCCCCCC/C=C\\\\C/C=C\\\\CCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCC/C=C\\\\C/C=C\\\\CCCCC)COC(=O)CCCCCCCCCCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCC/C=C\\\\CCCCCCCC)COC(=O)CCCCCCCCCCCCCCCCCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCC/C=C\\\\CCCCCCCC)COC(=O)CC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found'), "
               "('P(OC[C@H](OC(=O)CCCC(O)/C=C/C(O)=O)COC(=O)CCCCCCCCCCCCCCC)(O)(O)=O', "
               "'No proper glycerol backbone found')]",
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 5,
    'num_false_positives': 100,
    'num_true_negatives': 49316,
    'num_false_negatives': 44,
    'num_negatives': None,
    'precision': 0.047619047619047616,
    'recall': 0.10204081632653061,
    'f1': 0.06493506493506494,
    'accuracy': 0.997088850702517}


"""
Classifies: CHEBI:26848 tannin
"""
from rdkit import Chem
from rdkit.Chem import Descriptors

def is_tannin(smiles: str):
    """
    Determines if a molecule is a tannin.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a tannin, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of polyphenolic structure
    num_aromatic_rings = Chem.Descriptors.AromaticRingCount(mol)
    if num_aromatic_rings < 2:
        return False, "Not enough aromatic rings (polyphenolic structure)"

    # Check for the presence of hydroxyl (-OH) groups
    num_hydroxyls = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'O' and atom.GetHybridization() == Chem.HybridizationType.SP3)
    if num_hydroxyls < 5:
        return False, "Not enough hydroxyl groups"

    # Check for the presence of ester or ether linkages
    num_esters = sum(1 for bond in mol.GetBonds() if bond.GetBondType() == Chem.BondType.ESTERS)
    num_ethers = sum(1 for bond in mol.GetBonds() if bond.GetBondType() == Chem.BondType.ETHER)
    if num_esters + num_ethers < 2:
        return False, "Not enough ester or ether linkages"

    # Check for the presence of glucosides (sugar moieties)
    num_sugar_rings = sum(1 for ring in mol.GetRingInfo().AtomRings() if len(ring) == 5 or len(ring) == 6)
    if num_sugar_rings > 0:
        return True, "Contains sugar moiety (glucoside)"

    # If all criteria are met, classify as a tannin
    return True, "Polyphenolic structure with hydroxyl groups and ester/ether linkages"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26848',
                          'name': 'tannin',
                          'definition': 'Any of a group of astringent '
                                        'polyphenolic vegetable principles or '
                                        'compounds, chiefly complex glucosides '
                                        'of catechol and pyrogallol.',
                          'parents': ['CHEBI:26195']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "module 'rdkit.Chem.Descriptors' has no attribute "
             "'AromaticRingCount'",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:26829 sulfoglycolipid
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_sulfoglycolipid(smiles: str):
    """
    Determines if a molecule is a sulfoglycolipid (a sulfate ester of a glycolipid).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a sulfoglycolipid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for presence of sulfate group
    has_sulfate = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'S' and atom.GetTotalDegree() == 4:
            neighbors = [mol.GetAtomWithIdx(n.GetIdx()).GetAtomicNum() for n in atom.GetNeighbors()]
            if sum(n == 8 for n in neighbors) == 4:
                has_sulfate = True
                break
    if not has_sulfate:
        return False, "No sulfate group found"

    # Check for presence of glycolipid
    glycosidic_bonds = []
    for bond in mol.GetBonds():
        if bond.GetBondType() == Chem.BondType.SINGLE:
            atom1 = bond.GetBeginAtom()
            atom2 = bond.GetEndAtom()
            if atom1.GetSymbol() == 'O' and atom2.GetSymbol() == 'C':
                glycosidic_bonds.append(bond)
            elif atom2.GetSymbol() == 'O' and atom1.GetSymbol() == 'C':
                glycosidic_bonds.append(bond)
    if not glycosidic_bonds:
        return False, "No glycosidic bonds found"

    # Check if sulfate group is attached to glycolipid
    for bond in glycosidic_bonds:
        atom1, atom2 = bond.GetBeginAtom(), bond.GetEndAtom()
        for neighbor in atom1.GetNeighbors() + atom2.GetNeighbors():
            if neighbor.GetSymbol() == 'S' and neighbor.GetTotalDegree() == 4:
                neighbors = [mol.GetAtomWithIdx(n.GetIdx()).GetAtomicNum() for n in neighbor.GetNeighbors()]
                if sum(n == 8 for n in neighbors) == 4:
                    return True, "Sulfate ester of a glycolipid"

    return False, "Sulfate group not attached to glycolipid"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26829',
                          'name': 'sulfoglycolipid',
                          'definition': 'A sulfate ester of a glycolipid.',
                          'parents': [   'CHEBI:33563',
                                         'CHEBI:35724',
                                         'CHEBI:61384']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': 'Attempt failed: Python argument types in\n'
               '    Mol.GetAtomWithIdx(Mol, Atom)\n'
               'did not match C++ signature:\n'
               '    GetAtomWithIdx(RDKit::ROMol {lvalue} self, unsigned int '
               'idx)',
    'attempt': 4,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 4,
    'num_false_positives': 100,
    'num_true_negatives': 8561,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.038461538461538464,
    'recall': 1.0,
    'f1': 0.07407407407407407,
    'accuracy': 0.9884593190998269}


"""
Classifies: CHEBI:25238 methoxybenzoic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_methoxybenzoic_acid(smiles: str):
    """
    Determines if a molecule is a methoxybenzoic acid (benzoic acid with one or more methoxy substituents).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a methoxybenzoic acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for presence of carboxylic acid group (-COOH)
    carboxylic_acid_pattern = Chem.MolFromSmarts('[CX3](=O)[OX2H1]')
    if not mol.HasSubstructMatch(carboxylic_acid_pattern):
        return False, "No carboxylic acid group found"

    # Check for presence of methoxy group (-OCH3) connected to benzene ring
    methoxy_benzene_pattern = Chem.MolFromSmarts('c-[OX2][CH3]')
    if not mol.HasSubstructMatch(methoxy_benzene_pattern):
        return False, "No methoxy group attached to benzene ring found"

    # Find all carboxylic acid groups
    carboxylic_matches = mol.GetSubstructMatches(carboxylic_acid_pattern)
    
    # Find all methoxy groups
    methoxy_matches = mol.GetSubstructMatches(methoxy_benzene_pattern)

    # For each carboxylic acid group, check if it's connected to a benzene ring
    # that also has a methoxy substituent
    for acid_match in carboxylic_matches:
        acid_carbon = acid_match[0]  # Get the carbon atom of COOH
        
        # Get the atom connected to COOH
        acid_neighbors = mol.GetAtomWithIdx(acid_carbon).GetNeighbors()
        for neighbor in acid_neighbors:
            if neighbor.GetIsAromatic():  # If COOH is connected to aromatic ring
                ring_atom = neighbor.GetIdx()
                
                # Find the ring this atom belongs to
                ring_info = mol.GetRingInfo()
                for ring in ring_info.AtomRings():
                    if ring_atom in ring and len(ring) == 6:  # Found 6-membered ring containing COOH
                        # Check if any methoxy group is attached to this ring
                        for methoxy_match in methoxy_matches:
                            methoxy_oxygen = methoxy_match[0]
                            methoxy_neighbors = mol.GetAtomWithIdx(methoxy_oxygen).GetNeighbors()
                            for methoxy_neighbor in methoxy_neighbors:
                                if methoxy_neighbor.GetIsAromatic():
                                    if methoxy_neighbor.GetIdx() in ring:
                                        # Count number of methoxy groups on this ring
                                        methoxy_count = sum(1 for m in methoxy_matches 
                                                          if any(mol.GetAtomWithIdx(m[0]).GetNeighbors()[i].GetIdx() in ring 
                                                                for i in range(len(mol.GetAtomWithIdx(m[0]).GetNeighbors()))))
                                        return True, f"Found benzoic acid with {methoxy_count} methoxy substituent{'s' if methoxy_count > 1 else ''}"

    return False, "No methoxy group found on the same ring as carboxylic acid"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25238',
                          'name': 'methoxybenzoic acid',
                          'definition': 'Any benzoic acid carrying one or more '
                                        'methoxy substituents.',
                          'parents': ['CHEBI:22723', 'CHEBI:51683']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 10,
    'num_false_positives': 100,
    'num_true_negatives': 107316,
    'num_false_negatives': 6,
    'num_negatives': None,
    'precision': 0.09090909090909091,
    'recall': 0.625,
    'f1': 0.15873015873015875,
    'accuracy': 0.9990133293618289}


"""
Classifies: CHEBI:22323 alkyl group
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_alkyl_group(smiles: str):
    """
    Determines if a molecule is an alkyl group (-CnH2n+1).
    
    Args:
        smiles (str): SMILES string of the molecule, with * representing attachment point
        
    Returns:
        bool: True if molecule is an alkyl group, False otherwise
        str: Reason for classification
    """
    # Check for presence of attachment point (*)
    if '*' not in smiles:
        return False, "No attachment point (*) found"
        
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Count atoms
    num_carbons = 0
    num_hydrogens = 0
    num_stars = 0
    num_other = 0
    
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C':
            num_carbons += 1
        elif atom.GetSymbol() == 'H':
            num_hydrogens += 1
        elif atom.GetSymbol() == '*':
            num_stars += 1
        else:
            num_other += 1
            
    # Should only contain C, H and one attachment point
    if num_other > 0:
        return False, "Contains atoms other than C, H and attachment point"
    if num_stars != 1:
        return False, "Must have exactly one attachment point"
        
    # Check for saturation - no double/triple bonds
    for bond in mol.GetBonds():
        if bond.GetBondType() != Chem.BondType.SINGLE:
            return False, "Contains unsaturated bonds"
            
    # Add implicit hydrogens to get total H count
    num_hydrogens += sum(atom.GetNumImplicitHs() for atom in mol.GetAtoms())
    
    # Check formula CnH2n+1
    if num_hydrogens != (2 * num_carbons + 1):
        return False, f"Does not match CnH2n+1 formula. Has C{num_carbons}H{num_hydrogens}"
        
    # Check connectivity - should be one continuous chain
    fragments = Chem.GetMolFrags(mol)
    if len(fragments) > 1:
        return False, "Structure is not a single connected group"
        
    return True, f"Valid alkyl group with {num_carbons} carbons"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22323',
                          'name': 'alkyl group',
                          'definition': 'A univalent group -CnH2n+1 derived '
                                        'from an alkane by removal of a '
                                        'hydrogen atom from any carbon atom.',
                          'parents': ['CHEBI:33248']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 4,
    'num_false_positives': 4,
    'num_true_negatives': 183891,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.5,
    'recall': 1.0,
    'f1': 0.6666666666666666,
    'accuracy': 0.9999782489301192}


"""
Classifies: CHEBI:138141 17alpha-hydroxy-C21-steroid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_17alpha_hydroxy_C21_steroid(smiles: str):
    """
    Determines if a molecule is a 17alpha-hydroxy-C21-steroid.

    A 17alpha-hydroxy-C21-steroid is defined as:
    Any C21-steroid carrying a hydroxy substituent at the 17alpha-position. 
    Note that individual examples may have ring substituents at other positions 
    and/or contain double bonds, aromatic A-rings, expanded/contracted rings etc., 
    so the formula and mass may vary from that given for the generic structure.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a 17alpha-hydroxy-C21-steroid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check number of atoms
    if mol.GetNumAtoms() != 21:
        return False, "Molecule does not have 21 atoms (not a C21-steroid)"

    # Check for 17alpha-hydroxy group
    atom_17 = mol.GetAtomWithIdx(16)  # 17th atom is at index 16 in RDKit
    if atom_17.GetAtomicNum() != 8 or not atom_17.GetIsAromatic():  # Oxygen atom, not aromatic
        return False, "No hydroxy group at position 17alpha"

    # Check for steroid backbone
    ring_info = mol.GetRingInfo()
    rings = ring_info.AtomRings()
    if len(rings) < 4:
        return False, "Less than 4 rings (not a steroid)"

    # Check for cyclopentane rings
    cyclopentane_rings = [ring for ring in rings if len(ring) == 5]
    if len(cyclopentane_rings) != 3:
        return False, "Not 3 cyclopentane rings (not a steroid)"

    # Check for cyclohexane ring
    cyclohexane_rings = [ring for ring in rings if len(ring) == 6]
    if len(cyclohexane_rings) != 1:
        return False, "Not 1 cyclohexane ring (not a steroid)"

    # If all checks pass, it's a 17alpha-hydroxy-C21-steroid
    return True, "17alpha-hydroxy-C21-steroid"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:138141',
                          'name': '17alpha-hydroxy-C21-steroid',
                          'definition': 'Any C21-steroid carrying a hydroxy '
                                        'substituent at the 17alpha-position. '
                                        'Note that individual examples may '
                                        'have ring substituents at other '
                                        'positions and/or contain double '
                                        'bonds, aromatic A-rings, '
                                        'expanded/contracted rings etc., so '
                                        'the formula and mass may vary from '
                                        'that given for the generic structure.',
                          'parents': ['CHEBI:35342', 'CHEBI:61313']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183925,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999945630307842}


"""
Classifies: CHEBI:24963 ketoaldonic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.AllChem import GetMorganFingerprintAsBitVect

def is_ketoaldonic_acid(smiles: str):
    """
    Determines if a molecule is a ketoaldonic acid.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a ketoaldonic acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check for carboxylic acid group
    carboxylic_acid_pattern = Chem.MolFromSmarts('[CX3](=O)[OX2H1]')
    if not mol.HasSubstructMatch(carboxylic_acid_pattern):
        return False, "No carboxylic acid group found"

    # Check for ketone group
    ketone_pattern = Chem.MolFromSmarts('[#6][CX3](=O)[#6]')
    if not mol.HasSubstructMatch(ketone_pattern):
        return False, "No ketone group found"
        
    # Check for hydroxyl groups
    hydroxyl_pattern = Chem.MolFromSmarts('[OX2H1]')
    hydroxyl_matches = mol.GetSubstructMatches(hydroxyl_pattern)
    if len(hydroxyl_matches) < 2:  # Need at least 2 hydroxyl groups (not counting COOH)
        return False, "Insufficient hydroxyl groups"
        
    # Check carbon chain length
    carbon_chain = Chem.MolFromSmarts('C-C-C')  # At least 3 carbons needed
    if not mol.HasSubstructMatch(carbon_chain):
        return False, "Carbon chain too short"

    # Check if molecule has expected formula pattern for ketoaldonic acid
    num_carbons = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    num_oxygens = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'O')
    
    # Ketoaldonic acids should have n carbons and n+1 oxygens (where n>=3)
    if num_carbons < 3 or num_oxygens != num_carbons + 1:
        return False, "Incorrect molecular formula pattern for ketoaldonic acid"

    # Additional check to ensure linear chain structure 
    ring_info = mol.GetRingInfo()
    if ring_info.NumRings() > 0:
        return False, "Contains rings - should be linear chain"

    # Check that the ketone is not at the carboxylic acid end
    ketone_matches = mol.GetSubstructMatches(ketone_pattern)
    carboxyl_matches = mol.GetSubstructMatches(carboxylic_acid_pattern)
    
    if ketone_matches and carboxyl_matches:
        ketone_carbon = ketone_matches[0][1]  # Get the central carbon of ketone
        carboxyl_carbon = carboxyl_matches[0][0]  # Get the carbon of COOH
        
        # If ketone carbon is directly connected to carboxyl carbon, reject
        if ketone_carbon == carboxyl_carbon:
            return False, "Ketone group cannot be part of carboxylic acid group"
            
        # Check for alpha-keto acid pattern
        alpha_keto_pattern = Chem.MolFromSmarts('O=C-C(=O)O')
        if mol.HasSubstructMatch(alpha_keto_pattern):
            return False, "Alpha-keto acid pattern not allowed"

    # Check for branching - should be mostly linear
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C' and len([n for n in atom.GetNeighbors()]) > 4:
            return False, "Too much branching for ketoaldonic acid"

    return True, "Valid ketoaldonic acid structure found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24963',
                          'name': 'ketoaldonic acid',
                          'definition': 'Oxo carboxylic acids formally derived '
                                        'from aldonic acids by replacement of '
                                        'a secondary CHOH group by a carbonyl '
                                        'group.',
                          'parents': ['CHEBI:33720', 'CHEBI:35381']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.13333333333333333 is too low.\n'
               "True positives: [('OC[C@H](C(C(O)=O)=O)O', 'Molecule contains "
               'required ketoaldonic acid features: carboxylic acid, ketone '
               "group, multiple hydroxyl groups in linear chain'), "
               "('OCC(=O)[C@@H](O)[C@H](O)[C@@H](O)C(O)=O', 'Molecule contains "
               'required ketoaldonic acid features: carboxylic acid, ketone '
               "group, multiple hydroxyl groups in linear chain')]\n"
               "False positives: [('C([C@@](CC(C(O)=O)=O)(C(=O)O)O)C(O)=O', "
               "'Molecule contains required ketoaldonic acid features: "
               'carboxylic acid, ketone group, multiple hydroxyl groups in '
               "linear chain'), ('O[C@H](CC(=O)C(O)=O)[C@H](O)C(O)=O', "
               "'Molecule contains required ketoaldonic acid features: "
               'carboxylic acid, ketone group, multiple hydroxyl groups in '
               "linear chain'), ('OC(=O)CC(O)(CC(=O)C(O)=O)C(O)=O', 'Molecule "
               'contains required ketoaldonic acid features: carboxylic acid, '
               "ketone group, multiple hydroxyl groups in linear chain'), "
               "('O[C@@H](CC(=O)C(O)=O)[C@H](O)C(O)=O', 'Molecule contains "
               'required ketoaldonic acid features: carboxylic acid, ketone '
               "group, multiple hydroxyl groups in linear chain'), "
               "('OC[C@@H](O)[C@@H](O)C(=O)[C@@H](O)C(O)=O', 'Molecule "
               'contains required ketoaldonic acid features: carboxylic acid, '
               "ketone group, multiple hydroxyl groups in linear chain'), "
               "('C(O)C(O)C(O)C(O)C(=O)C(O)=O', 'Molecule contains required "
               'ketoaldonic acid features: carboxylic acid, ketone group, '
               "multiple hydroxyl groups in linear chain'), "
               "('O[C@H](CC(=O)C(O)=O)C(O)=O', 'Molecule contains required "
               'ketoaldonic acid features: carboxylic acid, ketone group, '
               "multiple hydroxyl groups in linear chain'), "
               "('OCC(=O)[C@@H](O)[C@H](O)C(=O)C(O)=O', 'Molecule contains "
               'required ketoaldonic acid features: carboxylic acid, ketone '
               "group, multiple hydroxyl groups in linear chain'), "
               "('OC(=O)CC(C(O)=O)C(=O)C(O)=O', 'Molecule contains required "
               'ketoaldonic acid features: carboxylic acid, ketone group, '
               "multiple hydroxyl groups in linear chain'), "
               "('OC[C@@H](O)[C@@H](O)[C@H](O)C(=O)C(O)=O', 'Molecule contains "
               'required ketoaldonic acid features: carboxylic acid, ketone '
               "group, multiple hydroxyl groups in linear chain'), "
               "('OC(=O)C(=O)CC(=O)C(O)=O', 'Molecule contains required "
               'ketoaldonic acid features: carboxylic acid, ketone group, '
               "multiple hydroxyl groups in linear chain'), "
               "('OC(C(O)C(=O)CO)C(O)C(O)=O', 'Molecule contains required "
               'ketoaldonic acid features: carboxylic acid, ketone group, '
               "multiple hydroxyl groups in linear chain'), "
               "('[C@@H](C(C(O)=O)=O)([C@@H](CO)O)O', 'Molecule contains "
               'required ketoaldonic acid features: carboxylic acid, ketone '
               "group, multiple hydroxyl groups in linear chain'), "
               "('O[C@@H](CC(=O)C(O)=O)[C@@H](O)C(O)=O', 'Molecule contains "
               'required ketoaldonic acid features: carboxylic acid, ketone '
               "group, multiple hydroxyl groups in linear chain'), "
               "('OC(=O)CC(=O)C(O)=O', 'Molecule contains required ketoaldonic "
               'acid features: carboxylic acid, ketone group, multiple '
               "hydroxyl groups in linear chain'), ('OCC(=O)C(O)=O', 'Molecule "
               'contains required ketoaldonic acid features: carboxylic acid, '
               "ketone group, multiple hydroxyl groups in linear chain'), "
               "('OC(=O)C(=O)CCP(O)=O', 'Molecule contains required "
               'ketoaldonic acid features: carboxylic acid, ketone group, '
               "multiple hydroxyl groups in linear chain'), "
               "('OC[C@H](O)[C@H](O)[C@@H](O)C(=O)C(O)=O', 'Molecule contains "
               'required ketoaldonic acid features: carboxylic acid, ketone '
               "group, multiple hydroxyl groups in linear chain'), "
               "('O[C@H]([C@@H](O)C(=O)CO)[C@H](O)C(O)=O', 'Molecule contains "
               'required ketoaldonic acid features: carboxylic acid, ketone '
               "group, multiple hydroxyl groups in linear chain'), "
               "('OC(CC(=O)C(O)=O)C(O)=O', 'Molecule contains required "
               'ketoaldonic acid features: carboxylic acid, ketone group, '
               "multiple hydroxyl groups in linear chain'), "
               "('OCC(=O)[C@@H](O)[C@@H](O)[C@H](O)C(O)=O', 'Molecule contains "
               'required ketoaldonic acid features: carboxylic acid, ketone '
               "group, multiple hydroxyl groups in linear chain'), "
               "('OC(=O)C[C@H](C(O)=O)C(=O)C(O)=O', 'Molecule contains "
               'required ketoaldonic acid features: carboxylic acid, ketone '
               "group, multiple hydroxyl groups in linear chain'), "
               "('OCC(=O)[C@@H](O)[C@H](O)[C@H](O)C(O)=O', 'Molecule contains "
               'required ketoaldonic acid features: carboxylic acid, ketone '
               "group, multiple hydroxyl groups in linear chain'), "
               "('O[C@@H](CC(=O)C(O)=O)C(O)=O', 'Molecule contains required "
               'ketoaldonic acid features: carboxylic acid, ketone group, '
               "multiple hydroxyl groups in linear chain'), "
               "('OC[C@H](O)[C@@H](O)C(=O)C(=O)C(O)=O', 'Molecule contains "
               'required ketoaldonic acid features: carboxylic acid, ketone '
               "group, multiple hydroxyl groups in linear chain')]\n"
               "False negatives: [('OC[C@@H](O)[C@H](O)CC(=O)C(O)=O', "
               "'Incorrect molecular formula pattern for ketoaldonic acid')]",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 5,
    'num_true_negatives': 183899,
    'num_false_negatives': 2,
    'num_negatives': None,
    'precision': 0.16666666666666666,
    'recall': 0.3333333333333333,
    'f1': 0.2222222222222222,
    'accuracy': 0.9999619372835183}


import logging
from pathlib import Path
from typing import Iterator, Annotated, List, Union, Optional

from c3p.datamodel import ClassificationResult, SMILES_STRING
from c3p.generator import run_code
from c3p.programs import PROGRAM_DIR

logger = logging.getLogger(__name__)


def classify(smiles: Union[SMILES_STRING, List[SMILES_STRING]], program_directory: Optional[Path] = None, strict=False) -> Iterator[ClassificationResult]:
    """
    Classify a SMILES string

    Args:
        smiles: The SMILES string to classify
        program_directory: The directory containing the programs

    Returns:
        The classification result
    """
    # find all programs in path
    if program_directory is None:
        program_directory = PROGRAM_DIR
    programs = list(program_directory.glob("*.py"))
    logger.info(f"Found {len(programs)} programs in {program_directory}")
    smiles_list = [smiles] if isinstance(smiles, str) else smiles
    # load each program
    for program in programs:
        if program.name.startswith("__"):
            continue
        with open(program, "r") as f:
            code = f.read()
        found = False
        for line in code.split("\n"):
            # use re to check if matches ^def is_(.*)\(smiles: str):
            import re
            fn_match = re.match(r"^def (is_.*)\(smiles: str\):", line)
            if fn_match:
                found = True
                fn = fn_match.group(1)
                for smiles in smiles_list:
                    try:
                        for _, satisfies, reason, metadata in run_code(code, fn, [smiles], []):
                            cc = metadata.get("chemical_class", {})
                            metric = "precision" if satisfies else "recall"
                            yield ClassificationResult(
                                input_smiles=smiles,
                                class_id = cc.get("id", "-"),
                                class_name = cc.get("name", "-"),
                                is_match=satisfies,
                                reason=reason,
                                confidence=metadata.get(metric)
                            )
                    except Exception as e:
                        if strict:
                            raise e
                        logger.error(f"Error running {fn} in {program}: {e}")
                break
        if not found:
            raise ValueError(f"Could not find is_ function in {program}")




"""
Classifies: CHEBI:132215 heparin octasaccharide
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from typing import Tuple

def is_heparin_octasaccharide(smiles: str) -> Tuple[bool, str]:
    """
    Determines if a molecule is a heparin octasaccharide, defined as any amino octasaccharide obtained
    by either chemical or enzymatic cleavage of the polymeric glycosaminoglycan heparin.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is a heparin octasaccharide, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Count the number of atoms
    num_atoms = mol.GetNumAtoms()
    if num_atoms != 64:
        return False, f"Molecule contains {num_atoms} atoms, expected 64 for an octasaccharide"

    # Count the number of rings
    ring_info = mol.GetRingInfo()
    num_rings = len(ring_info.AtomRings())
    if num_rings != 8:
        return False, f"Molecule contains {num_rings} rings, expected 8 for an octasaccharide"

    # Check for the presence of sulfate groups
    sulfate_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == "S")
    if sulfate_count < 3:
        return False, "Molecule does not contain at least 3 sulfate groups, as expected for heparin"

    # Check for the presence of amino groups
    amino_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == "N")
    if amino_count < 2:
        return False, "Molecule does not contain at least 2 amino groups, as expected for heparin"

    return True, "The molecule is a heparin octasaccharide"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:132215',
                          'name': 'heparin octasaccharide',
                          'definition': 'Any amino octasaccharide obtained by '
                                        'either chemical or enzymatic cleavage '
                                        'of the polymeric glycosaminoglycan '
                                        'heparin.',
                          'parents': ['CHEBI:139173', 'CHEBI:71061']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183924,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999945630012234}


"""
Classifies: CHEBI:24401 glycosinolate
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_glycosinolate(smiles: str):
    """
    Determines if a molecule is a glycosinolate based on its SMILES string.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a glycosinolate, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Look for sulfate group [O-]S(=O)(=O)O
    sulfate_pattern = Chem.MolFromSmarts('[O,O-]S(=O)(=O)[O,O-]')
    if not mol.HasSubstructMatch(sulfate_pattern):
        return False, "Missing sulfate group"

    # Look for C=N-O-S pattern 
    cn_os_pattern = Chem.MolFromSmarts('[C]=[N]-[O]-[S]')
    if not mol.HasSubstructMatch(cn_os_pattern):
        return False, "Missing C=N-O-S linkage"

    # Look for pyranose sugar ring with S attachment
    sugar_pattern = Chem.MolFromSmarts('[C]1[O][C]([C]([O])[C]([O])[C]1[O])[S]')
    if not mol.HasSubstructMatch(sugar_pattern):
        return False, "Missing pyranose sugar ring with S attachment"

    # Look for thiohydroximate group
    thiohyd_pattern = Chem.MolFromSmarts('[S]-[C]=[N]-[O]-[S]')
    if not mol.HasSubstructMatch(thiohyd_pattern):
        return False, "Missing thiohydroximate group"
        
    # Check for negative charge
    if '[O-]' not in smiles and 'O-' not in smiles:
        return False, "Missing negatively charged oxygen"

    # If all patterns are found and has negative charge, it's a glycosinolate
    return True, "Contains glycosinolate core structure (sulfated thiohydroximate-O connected to pyranose sugar)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24401',
                          'name': 'glycosinolate',
                          'definition': 'An sulfur oxoanion resulting from the '
                                        'deprotonation of the hydroxy group '
                                        'attached to the sulfur of a '
                                        'glycosinolic acid.',
                          'parents': ['CHEBI:33482']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.21428571428571425 is too low.\n'
               'True positives: '
               "[('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS([O-])(=O)=O)/CCCCCSC', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('OC[C@H]1O[C@@H](S\\\\C(CCCCOC(=O)C2=CC=CC=C2)=N\\\\OS([O-])(=O)=O)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS([O-])(=O)=O)/CC2=CC(=CC=C2)OC', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS([O-])(=O)=O)/CCC=C', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('S(C1OC(C(O)C(O)C1O)CO)\\\\C(=N\\\\OS(O)(=O)=O)\\\\CCC2=CC=CC=C2', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('S([C@@H]1O[C@@H]([C@@H](O)[C@H](O)[C@H]1O)CO)\\\\C(=N\\\\OS(O)(=O)=O)\\\\CC2=CC=C(O)C=C2', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)')]\n"
               'False positives: '
               "[('CC(CO)C(S[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)=NOS(O)(=O)=O', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('OC[C@H]1O[C@@H](SC(Cc2cn(c3ccccc23)S(O)(=O)=O)=NOS(O)(=O)=O)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('CS(=O)CCCC(S[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)=NOS([O-])(=O)=O', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCCC=C', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('S([C@@H]1O[C@@H]([C@@H](O)[C@H](O)[C@H]1O)CO)C(=NOS(O)(=O)=O)CCCCCCS(C)=O', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CC(CC)C', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCCCCCCCCS(C)=O', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)SC(=NOS(O)(=O)=O)C[C@H](C=C)O', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCCCS(C)=O', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCCC', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CC2=CC(=CC=C2)OC', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/C', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/C(C)C', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/C(CC)C', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CC(C)C', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('CSCCCCCCC(S[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)=NOS(O)(=O)=O', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('OC[C@H]1O[C@@H](SC(CC(O)c2ccccc2)=NOS(O)(=O)=O)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('OC[C@H]1O[C@@H](SC(CC(O)CC=C)=NOS(O)(=O)=O)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCCSC', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCC=C', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCCCCSC', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('O1[C@@H]([C@H]([C@@H]([C@H]([C@@H]1SC(CCCCCCCS(C)=O)=NOS(O)(=O)=O)O)O)O)CO', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)SC(=NOS(O)(=O)=O)C[C@@H](C=C)O', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CC=2C=CC=CC2', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('COc1cccc2[nH]cc(CC(S[C@@H]3O[C@H](CO)[C@@H](O)[C@H](O)[C@H]3O)=NOS(O)(=O)=O)c12', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('CCC(C)(O)CC(S[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)=NOS(O)(=O)=O', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCCCC', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('COc1ccc(CC(S[C@@H]2O[C@H](CO)[C@@H](O)[C@H](O)[C@H]2O)=NOS(O)(=O)=O)cc1', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('OC[C@H]1O[C@@H](SC(Cc2c[nH]c3ccccc23)=NOS(O)(=O)=O)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCCCCS(C)=O', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCC', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('OC[C@H]1O[C@@H](SC(Cc2ccc(O)cc2)=NOS(O)(=O)=O)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('CS(=O)(=O)CCCCC(S[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)=NOS([O-])(=O)=O', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('COn1cc(CC(S[C@@H]2O[C@H](CO)[C@@H](O)[C@H](O)[C@H]2O)=NOS(O)(=O)=O)c2ccccc12', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('CC(C)(O)CC(S[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)=NOS(O)(=O)=O', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCCCSC', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/*', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('CS(=O)(=O)CCCC(S[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)=NOS([O-])(=O)=O', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('OC[C@H]1O[C@@H](SC(Cc2c[nH]c3cccc(O)c23)=NOS(O)(=O)=O)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CC', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCCCCCCCS(C)=O', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CCC=2C=CC=CC2', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)S/C(=N\\\\OS(O)(=O)=O)/CC=C', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)SC(=NOS(O)(=O)=O)CC(C=C)O', "
               "'Contains glycosinolate core structure (sulfated "
               "thiohydroximate-O connected to pyranose sugar)')]\n"
               'False negatives: []',
    'attempt': 2,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 4,
    'num_false_positives': 3,
    'num_true_negatives': 183870,
    'num_false_negatives': 2,
    'num_negatives': None,
    'precision': 0.5714285714285714,
    'recall': 0.6666666666666666,
    'f1': 0.6153846153846153,
    'accuracy': 0.9999728082053959}


"""
Classifies: CHEBI:25676 oligopeptide
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_oligopeptide(smiles: str):
    """
    Determines if a molecule is an oligopeptide (a peptide containing a relatively small number of amino acids).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an oligopeptide, False otherwise
        str: Reason for classification
    """
    
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Count amide bonds (peptide bonds)
    amide_pattern = Chem.MolFromSmarts('[NX3H1,NX3H0][CX3](=[OX1])[#6]')
    amide_matches = mol.GetSubstructMatches(amide_pattern)
    num_amide_bonds = len(amide_matches)
    
    if num_amide_bonds == 0:
        return False, "No peptide bonds found"
    
    # Look for amino acid characteristics
    amine_pattern = Chem.MolFromSmarts('[NX3;H2,H1;!$(NC=O)]')
    carboxyl_pattern = Chem.MolFromSmarts('[CX3](=O)[OX2H1,OX1-]')
    
    has_amine = len(mol.GetSubstructMatches(amine_pattern)) > 0
    has_carboxyl = len(mol.GetSubstructMatches(carboxyl_pattern)) > 0
    
    if not (has_amine and has_carboxyl):
        return False, "Missing characteristic amino acid groups"

    # Estimate number of amino acid residues based on peptide bonds
    estimated_residues = num_amide_bonds + 1
    
    # Define size limits for oligopeptides (typically 2-20 amino acids)
    MIN_RESIDUES = 2
    MAX_RESIDUES = 20
    
    if estimated_residues < MIN_RESIDUES:
        return False, f"Too few amino acid residues ({estimated_residues})"
    elif estimated_residues > MAX_RESIDUES:
        return False, f"Too many amino acid residues ({estimated_residues})"
        
    return True, f"Oligopeptide with approximately {estimated_residues} amino acid residues"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25676',
                          'name': 'oligopeptide',
                          'definition': 'A peptide containing a relatively '
                                        'small number of amino acids.',
                          'parents': ['CHEBI:16670']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 331,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 100,
    'num_negatives': None,
    'precision': 1.0,
    'recall': 0.7679814385150812,
    'f1': 0.868766404199475,
    'accuracy': 0.7679814385150812}


"""
Classifies: CHEBI:22080 TDP-sugar
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.AllChem import GetMorganFingerprintAsBitVect
from rdkit.Chem import rdMolDescriptors

def is_TDP_sugar(smiles: str):
    """
    Determines if a molecule is a TDP-sugar.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a TDP-sugar, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for thymine substructure
    thymine_pattern = Chem.MolFromSmarts('Cc1cn([C,c])c(=O)[nH]c1=O')
    if not mol.HasSubstructMatch(thymine_pattern):
        return False, "Missing thymine nucleobase"

    # Check for deoxyribose substructure connected to thymine
    deoxyribose_pattern = Chem.MolFromSmarts('[C,c]1[C,c][C,c](O)[C,c](CO)[O,o]1')
    if not mol.HasSubstructMatch(deoxyribose_pattern):
        return False, "Missing deoxyribose sugar"

    # Check for diphosphate linkage
    diphosphate_pattern = Chem.MolFromSmarts('OP(O)(=O)OP(O)(=O)O')
    if not mol.HasSubstructMatch(diphosphate_pattern):
        return False, "Missing diphosphate linkage"

    # Check for sugar moiety (looking for multiple OH groups and ring structure)
    # This is a simplified check for a sugar structure
    sugar_pattern = Chem.MolFromSmarts('[C,c]1[C,c][C,c][C,c][C,c][O,o]1')
    if not mol.HasSubstructMatch(sugar_pattern):
        return False, "Missing sugar moiety"

    # Count OH groups (excluding phosphate OH groups)
    oh_pattern = Chem.MolFromSmarts('[C,c]O')
    oh_matches = len(mol.GetSubstructMatches(oh_pattern))
    if oh_matches < 2:  # Most sugars have multiple OH groups
        return False, "Insufficient hydroxyl groups for sugar moiety"

    # Check connectivity - the diphosphate should connect the deoxyribose to the sugar
    # This is a complex check that would require more sophisticated analysis
    # Here we just verify basic structural elements are present

    return True, "Contains thymine, deoxyribose, diphosphate linkage and sugar moiety"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22080',
                          'name': 'TDP-sugar',
                          'definition': 'A pyrimidine nucleotide-sugar having '
                                        'TDP as the nucleotide component '
                                        'attached to an unspecified sugar via '
                                        'an anomeric diphosphate linkage.',
                          'parents': ['CHEBI:61109']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 5,
    'num_false_positives': 86,
    'num_true_negatives': 183780,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.054945054945054944,
    'recall': 0.8333333333333334,
    'f1': 0.10309278350515463,
    'accuracy': 0.9995268447615733}


"""
Classifies: CHEBI:166988 glycerophosphoglycerophosphoglycerol
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_glycerophosphoglycerophosphoglycerol(smiles: str):
    """
    Determines if a molecule is a glycerophosphoglycerophosphoglycerol.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a glycerophosphoglycerophosphoglycerol, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for presence of required elements
    atoms = [atom.GetSymbol() for atom in mol.GetAtoms()]
    if not all(element in atoms for element in ['C', 'O', 'P']):
        return False, "Missing required elements (C, O, P)"

    # Count phosphorus atoms - should be exactly 2
    p_count = len([atom for atom in mol.GetAtoms() if atom.GetSymbol() == 'P'])
    if p_count != 2:
        return False, f"Found {p_count} phosphorus atoms, expected 2"

    # Core structure pattern: two phosphate groups connected by a glycerol bridge
    core_pattern = Chem.MolFromSmarts('[P](=O)([O,OH])[O]CC([OH1,O])COP(=O)([O,OH])[O]')
    if not mol.HasSubstructMatch(core_pattern):
        return False, "Missing core glycerophosphoglycerophosphoglycerol structure"

    # Check for glycerol moieties with ester linkages
    glycerol_ester_pattern = Chem.MolFromSmarts('[O]P(=O)([O,OH])[O]CC(OC(=O)[C])CO[C](=O)[C]')
    if len(mol.GetSubstructMatches(glycerol_ester_pattern)) < 2:
        return False, "Missing required glycerol moieties with ester linkages"

    # Check for fatty acid chains (at least 8 carbons)
    fatty_acid_pattern = Chem.MolFromSmarts('[C][C][C][C][C][C][C][C]')
    fatty_acid_count = len(mol.GetSubstructMatches(fatty_acid_pattern))
    if fatty_acid_count < 2:
        return False, "Missing required fatty acid chains"

    # Additional check for the complete structure connectivity
    complete_pattern = Chem.MolFromSmarts('[O]P(=O)([O,OH])[O]CC(OC(=O)[C])CO[C](=O)[C].CC([OH1,O])COP(=O)([O,OH])[O]CC(OC(=O)[C])CO[C](=O)[C]')
    if not mol.HasSubstructMatch(complete_pattern):
        return False, "Incorrect overall structure connectivity"

    return True, "Valid glycerophosphoglycerophosphoglycerol structure"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:166988',
                          'name': 'glycerophosphoglycerophosphoglycerol',
                          'definition': 'A glycerophospholipid composed of two '
                                        'molecules of glycerol phosphate '
                                        'covalently linked to a molecule of '
                                        'glycerol, and in which each of the '
                                        'glycerol phosphate moieties may be '
                                        'esterified to one or two fatty acids.',
                          'parents': ['CHEBI:37739']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0 is too low.\n'
               'True positives: []\n'
               'False positives: []\n'
               'False negatives: '
               "[('P(OC[C@H](OC(=O)CCCCCCCCCC)COC(=O)CCCCCCCCC)(OC[C@@H](O)COP(OC[C@H](OC(=O)CCCCCCCCC)COC(=O)CCCCCCCCC)(O)=O)(O)=O', "
               "'Missing required phosphate groups'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCC)COC(=O)CCCCCCCCCCCCCCCCC)(OC[C@H](O)COP(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCCCC)COC(=O)CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)(O)=O)(O)=O', "
               "'Missing required phosphate groups'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCC)COC(=O)CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)(OC[C@@H](O)COP(OC[C@H](OC(=O)CCCCCCC/C=C\\\\CCCCCCCC)COC(=O)CCCCCCCCCCCCCCC)(O)=O)(O)=O', "
               "'Missing required phosphate groups'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCC)COC(=O)CCCCCCC/C=C\\\\C/C=C\\\\CCCCC)(OC[C@H](O)COP(OC[C@H](OC(=O)CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)COC(=O)CCCCCCC/C=C\\\\CCCCCCCC)(O)=O)(O)=O', "
               "'Missing required phosphate groups')]",
    'attempt': 2,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 4,
    'num_false_positives': 11,
    'num_true_negatives': 183887,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.26666666666666666,
    'recall': 1.0,
    'f1': 0.4210526315789474,
    'accuracy': 0.9999401855335994}


"""
Classifies: CHEBI:18331 keratan 6'-sulfate
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_keratan_6__sulfate(smiles: str):
    """
    Determines if a molecule is a keratan 6'-sulfate (keratan sulfate with random sulfation at 6'-position)

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a keratan 6'-sulfate, False otherwise
        str: Reason for classification
    """
    # This is a complex carbohydrate structure that requires advanced pattern matching
    # Since keratan sulfate is a complex polysaccharide with specific sulfation patterns,
    # accurate classification would require detailed structural analysis beyond basic SMILES parsing
    # Therefore returning None to indicate this is beyond current capabilities
    return None, None


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:18331',
                          'name': "keratan 6'-sulfate",
                          'definition': 'A keratan sulfate with random '
                                        "sulfation at the 6'-position.",
                          'parents': ['CHEBI:60924']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183648,
    'num_false_negatives': 29,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9998421141460281}


"""
Classifies: CHEBI:142772 Aspidosperma alkaloid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_Aspidosperma_alkaloid(smiles: str):
    """
    Determines if a molecule is an Aspidosperma alkaloid.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an Aspidosperma alkaloid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for at least one indole ring
    indole_ring = Chem.MolFromSmarts('c1ccc2c(c1)cnc2')
    is_indole = mol.HasSubstructMatch(indole_ring)
    if not is_indole:
        return False, "No indole ring found"

    # Check for tryptamine substructure
    tryptamine = Chem.MolFromSmarts('c1ccc2c(c1)cnc2CCNC')
    is_tryptamine = mol.HasSubstructMatch(tryptamine)
    if not is_tryptamine:
        return False, "No tryptamine substructure found"

    # Check for C19 or C20 skeleton
    num_atoms = mol.GetNumAtoms()
    if num_atoms not in [19, 20]:
        return False, "Incorrect number of atoms for Aspidosperma alkaloid"

    # Check for seconloganin-derived C9 or C10 unit
    seconloganin_c9 = Chem.MolFromSmarts('C1CCC2C(C1)CC(CO2)C')
    seconloganin_c10 = Chem.MolFromSmarts('C1CCC2C(C1)CC(CO2)CC')
    is_seconloganin = mol.HasSubstructMatch(seconloganin_c9) or mol.HasSubstructMatch(seconloganin_c10)
    if not is_seconloganin:
        return False, "No seconloganin-derived C9 or C10 unit found"

    return True, "Aspidosperma alkaloid detected"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:142772',
                          'name': 'Aspidosperma alkaloid',
                          'definition': 'Any member of a group of indole '
                                        'alkaloids based on a C19 or C20 '
                                        'skeleton formed by condensation of '
                                        'tryptamine with a rearranged '
                                        'seconloganin-derived C9 or C10 unit. '
                                        'The Aspidosperma alkaloids constitute '
                                        'the largest group of indole alkaloids '
                                        '(currently ca. 220 alkaloids) and may '
                                        'be divided into several sub-groups.',
                          'parents': ['CHEBI:38958']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183925,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999945630307842}


"""
Classifies: CHEBI:17334 penicillin
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_penicillin(smiles: str):
    """
    Determines if a molecule is a penicillin - containing the penam core structure with
    specific substituents at positions 2, 3 and 6.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a penicillin, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # SMARTS pattern for penam core with required substituents:
    # - Two methyl groups at position 2
    # - Carboxylate at position 3  
    # - Carboxamide at position 6
    penam_pattern = "[H][C@]12SC(C)(C)[C@@H](N1C(=O)[C@H]2NC(=O)[*])[C,c]([O,N,S])[!#1]"
    
    pattern_mol = Chem.MolFromSmarts(penam_pattern)
    if pattern_mol is None:
        return None, "Invalid SMARTS pattern"

    # Check if structure matches penam pattern
    matches = mol.GetSubstructMatches(pattern_mol)
    if not matches:
        return False, "Does not contain penam core structure with required substituents"

    # Additional checks could be added here for specific substituent patterns
    
    return True, "Contains penam core with required substituents at positions 2, 3 and 6"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:17334',
                          'name': 'penicillin',
                          'definition': 'Any member of the group of '
                                        'substituted penams containing two '
                                        'methyl substituents at position 2, a '
                                        'carboxylate substituent at position 3 '
                                        'and a carboxamido group at position '
                                        '6.',
                          'parents': ['CHEBI:25865']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183873,
    'num_false_negatives': 6,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999673698464752}


"""
Classifies: CHEBI:132474 fatty acid-taurine conjugate
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_fatty_acid_taurine_conjugate(smiles: str):
    """
    Determines if a molecule is a fatty acid-taurine conjugate.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a fatty acid-taurine conjugate, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Find the taurine moiety
    taurine_pattern = Chem.MolFromSmarts('NCCS(=O)(=O)O')
    taurine_match = mol.GetSubstructMatch(taurine_pattern)

    if not taurine_match:
        return False, "No taurine moiety found"

    # Find the fatty acid moiety
    fatty_acid_pattern = Chem.MolFromSmarts('C(=O)CCCCCCCCCCC')
    fatty_acid_match = mol.GetSubstructMatch(fatty_acid_pattern)

    if not fatty_acid_match:
        return False, "No fatty acid moiety found"

    # Check if the fatty acid and taurine moieties are connected
    taurine_idx = taurine_match[0]
    fatty_acid_idx = fatty_acid_match[0]

    if not mol.GetBondBetweenAtoms(taurine_idx, fatty_acid_idx):
        return False, "Fatty acid and taurine moieties are not connected"

    return True, "The molecule is a fatty acid-taurine conjugate"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:132474',
                          'name': 'fatty acid-taurine conjugate',
                          'definition': 'A monocarboxylic acid amide obtained '
                                        'by formal condensation of the carboxy '
                                        'group of any fatty acid with the '
                                        'amino group of taurine.',
                          'parents': ['CHEBI:29347', 'CHEBI:33551']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': 'Attempt failed: tuple index out of range',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 86,
    'num_true_negatives': 183837,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.011494252873563218,
    'recall': 1.0,
    'f1': 0.022727272727272724,
    'accuracy': 0.9995324155629499}